# -*- coding: utf-8 -*-

from pylab import *
import pygame
#==============================================================================
# pygame.mixer.pre_init(44100, -16, 1, 512)
# pygame.mixer.init()
#==============================================================================
def angryBird(m, g, v0, y0, alpha, diam, rho, k, time, dt):
    """ finds the trajectory of the redstripe with given initial values.
    
    Args:
        m     : mass of the redstripe in kg
        g     : gravity in m/s^2
        v0    : initial release velocity of the redstripe in m/s
        y0    : initial release height of the redstripe in meters
        alpha : initial release angle of the redstripe in degrees
        diam  : diameter of the redstripe in meters
        rho   : air density in kg/m^3
        k     : stiffnes of the spring N/m
        time  : simulation time
        dt    : time step.
    Returns:
        t     : time in seconds
        r     : position in meters
        v     : velocity in m/s
    Raises:
        TypeError: if n is not a number
        ValueError: if n is negative
    """
    alpha = radians(alpha)      # Initial angle in radian
    D = 3.0*rho*diam**2         # drag coefficient
    R = (diam/2)                # ball contact starts - equilibrium length of Spring
    # Numerical initialization
    n = int(ceil(time/dt))
    a = zeros((n, 2), float)
    v = zeros((n, 2), float)
    r = zeros((n, 2), float)
    t = zeros((n, 2), float)
    # Set initial values
    r[0,1] = y0                             # initial position of the ball
    v[0,:] = v0*cos(alpha), v0*sin(alpha)   # initial velocity in i, j

    # Integration loop
    for i in range(n-1):
        if (r[i,1] < R):
            N = k*(R-r[i,1])*array([0,1])
        else:
            N = array([0,0])
        FD = - D*norm(v[i,:])*v[i,:]
        G = -m*g*array([0,1])
        Fnet = N + FD + G
        a = Fnet/m
        v[i+1,:] = v[i] + a*dt
        r[i+1,:] = r[i] + v[i+1]*dt
        t[i+1] = t[i] + dt
    return t, r, v

# Call this function so the Pygame library can initialize itself
pygame.init()
# This sets the name of the window
pygame.display.set_caption('BCO611 is cool')
clock = pygame.time.Clock()
# Before the loop, load the sounds:
click_sound = pygame.mixer.Sound("voyyy.ogg")
fire_sound = pygame.mixer.Sound("boink.ogg")
# Set positions of graphics
background_position = [0, 0]
# Load and set up graphics.
background_image = pygame.image.load("background.png")
# Create an 1280x800 sized screen
screen = pygame.display.set_mode(background_image.get_rect().size)
background_image = background_image.convert()
#screen = pygame.display.set_mode([1280, 800])
background_image_ground = 600
player_image = pygame.image.load("red_sprite_small.png")
player_image_width, player_image_height = player_image.get_rect().size
player_image = player_image.convert()

mass = 4             # mass of the ball in kg
gravity = 9.81          # gravitational acceleration in m/sˆ2
v0 = 70                 # Initial velocity in m/s
#y0 = 0                 # Initial position in meters
alpha = 30              # Initial angle in degrees
diameter = 0.05         # diameter of the ball in meters
airdensity = 1.225      # air density in kg/m^3
stiffnes = 1000.0       # stiffnes of the spring N/m
time = 60.0             # simulation time
dt = 0.01               # time step

done = False
simulation = False

while not done:
    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            done = True
        elif event.type == pygame.MOUSEBUTTONDOWN:
            _, r, _ = angryBird(mass, gravity, v0, background_image_ground - y, alpha, diameter, airdensity, stiffnes, time, dt)
            click_sound.play()
            simulation = True
            
    # Copy image to screen:
    screen.blit(background_image, background_position)
    # Get the current mouse position. This returns the position
    # as a list of two numbers.
    if not simulation:
        player_position = pygame.mouse.get_pos()
        x = player_position[0] - player_image_width/2
        y = player_position[1] - player_image_height/2
        if y > background_image_ground:  y = background_image_ground
        # Copy image to screen:
        screen.blit(player_image, [x, y])
    else:
        pygame.mouse.set_visible(False)
        for i in range(len(r)):
            screen.blit(background_image, background_position)
            xsim = x + r[i, 0]
            ysim = background_image_ground - r[i, 1]
            if ysim >= background_image_ground: fire_sound.play()
            # Copy image to screen:
            screen.blit(player_image, [xsim, ysim])
            pygame.mouse.set_pos(xsim + 45 , ysim + 45)
            pygame.display.flip()
        pygame.mouse.set_visible(True)
        simulation = False
    
    pygame.display.flip()
    clock.tick(60)
    
pygame.quit ()
